package geniusweb.protocol.session;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.PartyId;
import geniusweb.deadline.DeadlineTime;
import geniusweb.issuevalue.Bid;
import geniusweb.progress.Progress;
import geniusweb.progress.ProgressTime;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.partyconnection.ConnectionWithParties;
import geniusweb.protocol.session.DefaultSessionState;
import geniusweb.protocol.session.SessionSettings;
import geniusweb.protocol.session.saop.SAOPSettings;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;

public class DefaultSessionStateTest extends GeneralTests<DefaultSessionState> {
	private final ObjectMapper jackson = new ObjectMapper();
	private DefaultSessionState1 state1, state1a, stateA, stateE, stateF;
	private String state1string = "{\"DefaultSessionState1\":{\"actions\":[],\"progress\":{\"time\":{\"duration\":1000,\"start\":1000}},\"settings\":{\"SAOPSettings\":{\"participants\":[{\"party\":\"party1\",\"profile\":\"profile1\"},{\"party\":\"party2\",\"profile\":\"profile2\"}],\"deadline\":{\"deadlinetime\":{\"durationms\":1000}}}},\"error\":null,\"partyprofiles\":{\"party1\":{\"party\":\"party1\",\"profile\":\"profile1\"}}}}";
	private ProgressTime progr;
	private final Date start = new Date(1000l);

	@Before
	public void before() throws URISyntaxException {
		List<Action> actions = new LinkedList<>();
		PartyId party1 = new PartyId("party1");
		actions.add(new EndNegotiation(party1));
		ConnectionWithParties conns = new ConnectionWithParties(
				Collections.EMPTY_LIST);
		DeadlineTime deadline = new DeadlineTime(1000);
		progr = new ProgressTime(deadline.getDuration(), start);
		PartyRef party1ref = new PartyRef("party1");
		ProfileRef profile1 = new ProfileRef("profile1");
		PartyRef party2ref = new PartyRef("party2");
		ProfileRef profile2 = new ProfileRef("profile2");
		List<PartyWithProfile> participants = Arrays.asList(
				new PartyWithProfile(party1ref, profile1),
				new PartyWithProfile(party2ref, profile2));
		SessionSettings settings = new SAOPSettings(participants, deadline);
		ProtocolException e = new ProtocolException("test protocol error",
				party1.getName());
		HashMap<PartyId, PartyWithProfile> partyprofiles = new HashMap<PartyId, PartyWithProfile>();
		partyprofiles.put(party1, new PartyWithProfile(party1ref, profile1));
		state1 = new DefaultSessionState1(Collections.EMPTY_LIST, null, progr,
				settings, partyprofiles, null);
		state1a = new DefaultSessionState1(Collections.EMPTY_LIST, null, progr,
				settings, partyprofiles, null);

		stateA = new DefaultSessionState1(actions, conns, progr, settings,
				partyprofiles, null);

		stateE = new DefaultSessionState1(Collections.EMPTY_LIST, conns, progr,
				settings, partyprofiles, e);
		stateF = new DefaultSessionState1(Collections.EMPTY_LIST, conns, progr,
				settings, null, e);

	}

	@Override
	public List<List<DefaultSessionState>> getGeneralTestData() {

		return Arrays.asList(Arrays.asList(state1, state1a),
				Arrays.asList(stateA), Arrays.asList(stateE),
				Arrays.asList(stateF));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"DefaultSessionState1.*\\[\\].*ProgressTime.*1000ms.*SAOPSettings.*party1.*party2.*,null]",
				"DefaultSessionState1.*\\[.*EndNegotiation.*party1.*\\].*,.*,ProgressTime.*1000ms.*SAOPSettings.*party1.*party2.*,null]",
				"DefaultSessionState1.*\\[\\],.*,ProgressTime.*1000ms.*,SAOPSettings.*party1.*party2.*,.*ProtocolException: party1:test protocol error.*",
				"DefaultSessionState1.*\\[\\],.*,ProgressTime.*1000ms.*,SAOPSettings.*party1.*party2.*,.*ProtocolException: party1:test protocol error.*");
	}

	@Test
	public void serializeTest() throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(state1));
		assertEquals(state1string, jackson.writeValueAsString(state1));
	}

	@Test
	public void deserializeTest() throws IOException {
		// note, we don't use DefaultSessionState because we can't add the
		// annotation nor tell jackson dynamically about the test class.
		DefaultSessionState state = jackson.readValue(state1string,
				DefaultSessionState1.class);
		// compare fields, as sessionended is a derived/new inner class
		assertEquals(progr, state.getProgress());
	}
}

/**
 * We need to implement a few things because we can not instantiate an abstract
 * class.
 */
class DefaultSessionState1 extends DefaultSessionState {

	private DefaultSessionState1() {
		super();
	}

	public DefaultSessionState1(List<Action> actions,
			ConnectionWithParties conns, Progress progr,
			SessionSettings settings,
			Map<PartyId, PartyWithProfile> partyprofiles, ProtocolException e) {
		super(actions, conns, progr, settings, partyprofiles, e);
	}

	@Override
	public Bid getAgreement() {
		return null;
	}

}
