package geniusweb.protocol.tournament;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigInteger;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.deadline.DeadlineTime;
import geniusweb.protocol.session.saop.SAOPSettings;
import geniusweb.protocol.tournament.TournamentSettings;
import geniusweb.protocol.tournament.allpermutations.AllPermutationsSettings;
import geniusweb.references.PartyRef;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;
import tudelft.utilities.logging.ReportToLogger;

public class AllPermutationsSettingsTest
		extends GeneralTests<AllPermutationsSettings> {

	private final ObjectMapper jackson = new ObjectMapper();

	private final String serialized = "{\"AllPermutationsSettings\":{"
			+ "\"parties\":[\"party1\",\"party2\"]," + "\"reuseParties\":false,"
			+ "\"profiles\":[\"profile1\",\"profile2\",\"profile3\"],"
			+ "\"partiesPerSession\":2,"
			+ "\"sessionsettings\":{\"SAOPSettings\":{\"participants\":[],\"deadline\":{\"deadlinetime\":{\"durationms\":10000}}}}}}";

	private AllPermutationsSettings settings, settings1a, settings2, settings3,
			settings4;

	private SAOPSettings saopsettings;

	private ProfileRef profile1, profile2, profile3;

	private List<PartyRef> parties;

	@Before
	public void before() throws URISyntaxException {
		PartyRef partyref1 = new PartyRef("party1");
		PartyRef partyref2 = new PartyRef("party2");
		PartyRef partyref3 = new PartyRef("party3");

		parties = Arrays.asList(partyref1, partyref2);
		DeadlineTime deadline = new DeadlineTime(10000);

		saopsettings = new SAOPSettings(Collections.emptyList(), deadline);

		profile1 = new ProfileRef("profile1");
		profile2 = new ProfileRef("profile2");
		profile3 = new ProfileRef("profile3");

		List<ProfileRef> profiles = Arrays.asList(profile1, profile2, profile3);

		settings = new AllPermutationsSettings(parties, false, profiles, 2,
				saopsettings);

		settings1a = new AllPermutationsSettings(parties, false, profiles, 2,
				saopsettings);

		settings2 = new AllPermutationsSettings(
				Arrays.asList(partyref1, partyref2, partyref3), false, profiles,
				2, saopsettings);

		settings3 = new AllPermutationsSettings(parties, true, profiles, 2,
				saopsettings);

		settings4 = new AllPermutationsSettings(parties, false,
				Arrays.asList(profile1, profile2, profile3, profile1), 2,
				saopsettings);

	}

	@Override
	public List<List<AllPermutationsSettings>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(settings, settings1a),
				Arrays.asList(settings2), Arrays.asList(settings3),
				Arrays.asList(settings4));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"AllPermutationsSettings.*party1.*party2.*,false,.*profile1.*profile2.*profile3.*,2,SAOPSettings.*",
				"AllPermutationsSettings.*party1.*party2.*party3.*,false,.*profile1.*profile2.*profile3.*,2,SAOPSettings.*",
				"AllPermutationsSettings.*party1.*party2.*,true,.*profile1.*profile2.*profile3.*,2,SAOPSettings.*",
				"AllPermutationsSettings.*party1.*party2.*,false,.*profile1.*profile2.*profile3.*profile1.*,2,SAOPSettings.*");
	}

	@Test
	public void getProtocolTest() {
		assertEquals("APP", settings.getProtocol(new ReportToLogger("test"))
				.getRef().getURI().toString());
	}

	@Test
	public void testDeserialize() throws IOException {
		TournamentSettings obj = jackson.readValue(serialized,
				TournamentSettings.class);
		System.out.println(obj);
		assertEquals(settings, obj);
	}

	@Test
	public void testSerialize()
			throws JsonProcessingException, URISyntaxException {

		String string = jackson.writeValueAsString(settings);
		System.out.println(string);
		assertEquals(serialized, string);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullParties() {
		new AllPermutationsSettings(null, true,
				Arrays.asList(profile1, profile2, profile3, profile1), 2,
				saopsettings);

	}

	@Test(expected = IllegalArgumentException.class)
	public void testNoParties() {
		new AllPermutationsSettings(Collections.emptyList(), true,
				Arrays.asList(profile1, profile2, profile3, profile1), 2,
				saopsettings);

	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullProfiles() {
		new AllPermutationsSettings(parties, true, null, 2, saopsettings);

	}

	@Test(expected = IllegalArgumentException.class)
	public void testInsufficientProfiles() {
		new AllPermutationsSettings(parties, true,
				Arrays.asList(profile1, profile2, profile3, profile1), 4,
				saopsettings);

	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullSessionSettings() {
		new AllPermutationsSettings(parties, true,
				Arrays.asList(profile1, profile2, profile3, profile1), 4, null);

	}

	@Test(expected = IllegalArgumentException.class)
	public void testOnePartyPerSession() {
		new AllPermutationsSettings(parties, true,
				Arrays.asList(profile1, profile2, profile3, profile1), 1,
				saopsettings);

	}

	@Test
	public void getMaxRuntimeTest() {
		assertEquals(BigInteger.valueOf(6), settings.permutations().size());
		assertEquals((Double) (6d * 12), settings.getMaxRunTime());
	}

}
